@@ -26,18 +26,21 @@
 
 #include "gtksourceview-i18n.h"
 
+/*
+ * Small hack since we don't have a proper place where
+ * do gettext initialization.
+ */
 char *
-gtksourceview_gettext (const char *msgid)
+_gtksourceview_gettext (const char *msgid)
 {
 	static gboolean initialized = FALSE;
 
-	if (!initialized) {
-		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	if (G_UNLIKELY (!initialized))
+	{
+		bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 		initialized = TRUE;
 	}
 
 	return dgettext (GETTEXT_PACKAGE, msgid);
 }
-
-
