@@ -4,14 +4,22 @@
 #ifndef __GTKSOURCEVIEW_TYPEBUILTINS_H__
 #define __GTKSOURCEVIEW_TYPEBUILTINS_H__ 1
 
+#include <gtksourceview/gtksourceiter.h>
 G_BEGIN_DECLS
-
-#include "gtksourceiter.h"
 #define GTK_TYPE_SOURCE_SEARCH_FLAGS gtk_source_search_flags_get_type()
 GType gtk_source_search_flags_get_type (void);
-#include "gtksourcetagstyle.h"
-#define GTK_TYPE_SOURCE_TAG_STYLE_MASK gtk_source_tag_style_mask_get_type()
-GType gtk_source_tag_style_mask_get_type (void);
+G_END_DECLS
+
+#include <gtksourceview/gtksourceview.h>
+G_BEGIN_DECLS
+#define GTK_TYPE_SOURCE_VIEW_SMART_HOME_END_TYPE gtk_source_view_smart_home_end_type_get_type()
+GType gtk_source_view_smart_home_end_type_get_type (void);
+G_END_DECLS
+
+#include <gtksourceview/gtksourcestyle.h>
+G_BEGIN_DECLS
+#define GTK_TYPE_SOURCE_STYLE_MASK gtk_source_style_mask_get_type()
+GType gtk_source_style_mask_get_type (void);
 G_END_DECLS
 
 #endif /* __GTKSOURCEVIEW_TYPEBUILTINS_H__ */
